/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.example;

import cz.tdp.kshield.client.UserInfo;
import cz.tdp.kshield.integration.SimpleAuthenticationServiceImpl;
import cz.tdp.kshield.integration.UserInfoValidator;

//SONAR-IGNORE-FILE

/**
 * Basic usage of AuthenticationService.<br/>
 * In real world scenarios is AuthenticationService used indirectly in Servlet Filter<br/>
 * AuthenticationService provides more precise control over authentication and client-server communication<br/>
 * <br/>
 * @see cz.tdp.kshield.integration.web.DefaultAuthenticationServiceImpl
 * @see cz.tdp.kshield.integration.web.KShieldAuthenticationFilter
 * @see cz.tdp.kshield.springsecurity2.KShieldPreAuthenticationFilter
 * @see cz.tdp.kshield.springsecurity3.KShieldPreAuthenticationFilter
 */
public class AuthServiceDemo
{
  public static void main(String[] args) {
    /*  first create authService  */
    
    // URL of KeyShield SSO server
    final String serverURL = "http://192.168.0.42:8485";
    
    // create authentication service
    final SimpleAuthenticationServiceImpl authService = new SimpleAuthenticationServiceImpl(serverURL);
    
    // set various attributes
    authService.setUsernameAttribute("cn");
    authService.setApiKey("ABCDEFGHIJKLMN");
    
    // userInfo validator is called after successfull userInfo creation, use it for custom validations (e.g. LDAP group check, hw token, etc.)
    authService.setUserInfoValidator(new UserInfoValidator() {
      @Override
      public boolean validate(UserInfo userInfo) {
        return userInfo.isHwTokenPresent();
      }
    });
    
    // initialize service - init method is automatically called in Spring Framework environment
    authService.init();
    
    
    /*  check user with given IP  */
    
    // IP address of user
    String ipAddress = "192.168.100.42";

    // get UserInfo for given IP address
    final UserInfo user = authService.createUserInfo(ipAddress);

    if(user != null) {
      System.out.print("authenticated user: " + user.getUserID());
    }
    else {
      System.out.print("No user authenticated at: "+ipAddress);
    }
    
    // freing resources - destroy method is automatically called in Spring Framework environment
    authService.destroy();
  }
}
