/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.integration.web;

import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.InitializingBean;

import cz.tdp.kshield.client.UserInfo;
import cz.tdp.kshield.integration.KShieldContext;
import cz.tdp.kshield.integration.ServletAuthenticationService;
import cz.tdp.kshield.integration.SimpleAuthenticationServiceImpl;

/**
 * Basic implementation of KeyShield SSO Server AuthenticationService.
 * KShieldClient is used internally to communicate with KeyShield SSO Server.
 * 
 * @see cz.tdp.kshield.client.KShieldClient
 */
public class DefaultAuthenticationServiceImpl extends SimpleAuthenticationServiceImpl implements ServletAuthenticationService, InitializingBean
{
  /**
   * @param url KeyShield SSO Server url
   */
  public DefaultAuthenticationServiceImpl(String url) {
    super(url);
  }
  
  @Override
  public boolean authenticate(HttpServletRequest request) {
    if (client == null) throw new IllegalStateException("This AuthenticaticationService must be initialized (method init) before attempt to authenticate");
    
    final String ipAddr = getClientIpAddress(request);
    
    if (ipAddr != null) {
      final UserInfo userInfo = createUserInfo(ipAddr);
      
      if (userInfo != null && doAuthentication(request, userInfo)) {
        // save principal in user session
        request.getSession().setAttribute(KSHIELD_PRINCIPAL_TOKEN, userInfo);
        return true;
      }
    }
    
    // close kshield session if authentication failed
    KShieldContext.closeKShieldSession();
    return false;
  }

  private String getClientIpAddress(HttpServletRequest request) {
    try {
      final InetAddress inetAddr = InetAddress.getByName(request.getRemoteAddr());
      final String ipAddr = inetAddr.getHostAddress();
      return ipAddr;
    }
    catch (UnknownHostException e) {
      log.error("Cannot determine client ip address from remoteAddr: " + request.getRemoteAddr(), e);
    }
    
    return null;
  }
  
  /**
   * Override this method to do real authentication steps
   * 
   * @param request current HTTP request
   * @param userInfo User data returned by KeyShield SSO Server client
   * @return true if user was sucessfully authenticated
   */
  protected boolean doAuthentication(HttpServletRequest request, UserInfo userInfo) {
    // only check if userInfo is not null
    return userInfo != null;
  }
  
  /* (non-Javadoc)
   * @see org.springframework.beans.factory.InitializingBean#afterPropertiesSet()
   */
  @Override
  public void afterPropertiesSet() {
    if (client == null) {
      init();
    }
  }

  private static final Log log = LogFactory.getLog(DefaultAuthenticationServiceImpl.class);
}
