/*
 * Copyright (c) 2013 - 2016 TDP Ltd All Rights Reserved.
 * TDP Ltd grants permission, free of charge, to any person obtaining copies
 * of this software and its associated documentation files (the "Software"),
 * to deal in the Software without restriction, including to use, copy, adapt,
 * publish, distribute, display, perform, sublicense, and sell copies of the
 * Software, subject to the following condition: You must include the above
 * copyright notice and this permission notice in all full or partial copies
 * of the Software.
 * 
 * TDP LTD PROVIDES THE SOFTWARE "AS IS," WITHOUT ANY EXPRESS OR IMPLIED WARRANTY,
 * INCLUDING WITHOUT THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, AND NON-INFRINGMENT. TDP LTD, THE AUTHORS OF THE SOFTWARE,
 * AND THE OWNERS OF COPYRIGHT IN THE SOFTWARE ARE NOT LIABLE FOR ANY CLAIM, DAMAGES,
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT, OR OTHERWISE, ARISING
 * FROM, OUT OF, OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
package cz.tdp.kshield.notification;

import cz.tdp.kshield.client.UserInfo;


/**
 * 
 */
public interface KShieldAuthManager
{
  /**
   * Call this after successfull kshield authentication
   * @param userInfo
   */
  void addAuth(UserInfo userInfo);
  
  /**
   * Call this after denied kshield authentication from given ip address
   * @param ipAddress
   */
  void removeAuth(String ipAddress);
  
  /**
   * Remove all info about authentication from given ip address
   * @param ipAddress
   */
  void resetAuth(String ipAddress);
  
  /**
   * Register custom handler called after successful kshield authentication
   * @param handler
   */
  void registerOnLoginHandler(AuthChangeHandler handler);
  
  /**
   * Register custom handler called after kshield authentication removal
   * @param handler
   */
  void registerOnLogoutHandler(AuthChangeHandler handler);
  
  /**
   * This method checks kshield server if authentication info is missing
   * 
   * @param ipAddress
   * @param userID
   * @return true if userID is authenticated by kshield from given ip address
   */
  boolean hasAuth(String ipAddress, String userID);
  
  /**
   * Call this after establishing connection with WebSocket Endpoint
   * 
   * @param ipAddress
   * @param ws WebSocket Endpoint instance
   */
  void addEndpoint(String ipAddress, KShieldEndpoint ws);
  
  /**
   * IMPORTANT: Call this after closing connection with WebSocket Endpoint
   * 
   * @param ipAddress
   * @param ws WebSocket Endpoint instance
   */
  void removeEndpoint(String ipAddress, KShieldEndpoint ws);
  
  /**
   * @param ipAddress
   * @param kshieldUserID userID returned by KeyShield SSO server
   * @param authenticated user is authenticated 
   * @return
   */
  String poll(String ipAddress, String kshieldUserID, boolean authenticated);
}
